-----------------------------------------------------------------------------

local L = AceLibrary("AceLocale-2.2"):new("AloftVisibility")

-----------------------------------------------------------------------------

L:RegisterTranslations("enUS", function() return {
	["Visibility"] = true,
	["Extra visibility options"] = true,
	["Show Vermin"] = true,
	["Shows nameplates for level 1 neutral units"] = true,
	["Show Hostile Players"] = true,
	["Shows nameplates for hostile players. Can take neutral units"] = true,
	["Show Friendly NPCs"] = true,
	["Show friendly NPCs"] = true,
	["Show Neutral Units"] = true,
	["Show neutral units"] = true,
	["Friendly Players"] = true,
	["Selects which friendly players to show"] = true,
} end)

-----------------------------------------------------------------------------

local AloftVisibility = Aloft:NewModule("Visibility", "AceEvent-2.0")
AloftVisibility.db = Aloft:AcquireDBNamespace("Visibility")
Aloft:RegisterDefaults("Visibility", "profile", {
	showVermin = true,
	showHostilePlayers = true,
	showFriendlyNPCs = true,
	showNeutralUnits = true,
	showGroupPets = true,
	showFriendlyPets = true,
	showHostilePets = true,
	friendlyPlayers = "All",
})

Aloft.Options.args.visibility =
{
	type = 'group',
	name = L["Visibility"],
	desc = L["Extra visibility options"],
	disabled = function() return not Aloft:IsActive() end,
	args =
	{
		showVermin =
		{
			type = 'toggle',
			name = L["Show Vermin"],
			desc = L["Shows nameplates for level 1 neutral units"],
			disabled = function() return not AloftVisibility.db.profile.showNeutralUnits end,
			get = function() return AloftVisibility.db.profile.showVermin end,
			set = function(v) AloftVisibility.db.profile.showVermin = v AloftVisibility:Update() end
		},
		showHostilePlayers =
		{
			type = 'toggle',
			name = L["Show Hostile Players"],
			desc = L["Shows nameplates for hostile players. Can take neutral units"],
			get = function() return AloftVisibility.db.profile.showHostilePlayers end,
			set = function(v) AloftVisibility.db.profile.showHostilePlayers = v Aloft:DetermineDataSources() AloftVisibility:Update() end
		},
		showFriendlyNPCs =
		{
			type = 'toggle',
			name = L["Show Friendly NPCs"],
			desc = L["Show friendly NPCs"],
			get = function() return AloftVisibility.db.profile.showFriendlyNPCs end,
			set = function(v) AloftVisibility.db.profile.showFriendlyNPCs = v AloftVisibility:Update() end
		},
		showNeutralUnits =
		{
			type = 'toggle',
			name = L["Show Neutral Units"],
			desc = L["Show neutral units"],
			get = function() return AloftVisibility.db.profile.showNeutralUnits end,
			set = function(v) AloftVisibility.db.profile.showNeutralUnits = v AloftVisibility:Update() end
		},
		friendlyPlayers =
		{
			type = 'text',
			name = L["Friendly Players"],
			desc = L["Selects which friendly players to show"],
			get = function() return AloftVisibility.db.profile.friendlyPlayers end,
			set = function(v) AloftVisibility.db.profile.friendlyPlayers = v Aloft:DetermineDataSources() AloftVisibility:Update() end,
			validate = {"All", "Guild Only", "Group Only", "None" },
		},
	},
}

-----------------------------------------------------------------------------

function AloftVisibility:Update()
	self:RegisterEvents()

	-- Toggle the visibility to get previously hidden stuff to show
	Aloft:RedisplayNameplates()
	Aloft:RedisplayFriendlyNameplates()
end

function AloftVisibility:RegisterEvents()
	self:UnregisterAllEvents()

	if not self.db.profile.showHostilePlayers then
		self:RegisterEvent("Aloft:OnClassDataChanged", "OnClassDataChanged")
	end
	
	if not self.db.profile.showVermin or 
	   not self.db.profile.showHostilePlayers or
	   not self.db.profile.showNeutralUnits or
	   not self.db.profile.showFriendlyNPCs or
	   self.db.profile.friendlyPlayers ~= "All" then
		self:RegisterEvent("Aloft:OnNameplateShow", "OnNameplateShow")
	end
end

-----------------------------------------------------------------------------

function AloftVisibility:OnEnable()	
	self:RegisterEvents()
end

function AloftVisibility:OnDisable()	
	self:UnregisterAllEvents()

	if not self.db.profile.showVermin or 
	   not self.db.profile.showHostilePlayers or
	   not self.db.profile.showNeutralUnits or
	   not self.db.profile.showFriendlyNPCs or
	   self.db.profile.friendlyPlayers ~= "All" then
		Aloft:RedisplayNameplates()
		Aloft:RedisplayFriendlyNameplates()
	end
end

local dataRequiredList = { }
function AloftVisibility:RequiresData()
	for i = 1,#dataRequiredList do
		dataRequiredList[i] = nil
	end

	if not self.db.profile.showHostilePlayers then
		table.insert(dataRequiredList, "class")
	end

	if self.db.profile.friendlyPlayers == "Guild Only" then
		table.insert(dataRequiredList, "isGuildMember")
	end

	return unpack(dataRequiredList)
end

-----------------------------------------------------------------------------

local friendlyPlayerMethods =
{
	["All"] = function() end,
	["Guild Only"] = function(frame, aloftData) if not aloftData.isGuildMember then frame:Hide() end end,
	["Group Only"] = function(frame, aloftData) if not aloftData.unitid then frame:Hide() end end,
	["None"] = function(frame) frame:Hide() end,
}

local showTypeMethods =
{
	["friendlyPlayer"] = function(frame, aloftData) 
							friendlyPlayerMethods[AloftVisibility.db.profile.friendlyPlayers](frame, aloftData)
						 end,
	["friendlyNPC"] = function(frame, aloftData) 
						if not AloftVisibility.db.profile.showFriendlyNPCs then 
							frame:Hide() 
						end
					  end,
	["neutral"] = function(frame, aloftData) 
					if not AloftVisibility.db.profile.showNeutralUnits or (not AloftVisibility.db.profile.showVermin and aloftData.level == 1) then 
						frame:Hide()
					end
				  end,
	["hostile"] = function(frame, aloftData)
					if not AloftVisibility.db.profile.showHostilePlayers and aloftData.class then 
						frame:Hide()
					end
				  end,
}

function AloftVisibility:OnNameplateShow(frame)
	local aloftData = frame.aloftData
	
	local showMethod = showTypeMethods[aloftData.type]
	if showMethod then
		showMethod(frame, aloftData)
	end
end

function AloftVisibility:OnClassDataChanged(frame)
	-- If it was changed, then it must have some value now
	if frame.aloftData.type == "hostile" then
		frame:Hide()
	end
end

-----------------------------------------------------------------------------
